@@ -37,6 +37,8 @@ impl<I: 'static + Send, O: 'static + Send, D: 'static + Demon<Input = I, Output
     }
 
     async fn fire(mut self) {
+        #[cfg(feature = "internal_log")]
+        log::debug!("[{}] demon thread starting", self.demon.id());
         // Inner message passing
         let (mailbox, mut messages) = mpsc::unbounded_channel::<(Sender<Result<Box<dyn Any + Send>, Error>>, Box<dyn Any + Send>)>();
         // We prepare out websockets buffer
@@ -48,23 +50,27 @@ impl<I: 'static + Send, O: 'static + Send, D: 'static + Demon<Input = I, Output
         self.demon.spawned(other_loc).await;
 
         loop {
+            #[cfg(feature = "internal_log")]
+            log::debug!("[{}] start of await loop", self.demon.id());
             tokio::select! {
                 res = messages.recv() => if let Some((tx, input)) = res {
                     if let Ok(input) = input.downcast::<I>() {
+                        #[cfg(feature = "internal_log")]
+                        log::warn!("[{}] demon handle processing message...", self.demon.id());
                         let output = self.demon.handle(*input).await;
                         if tx.send(Ok(Box::new(output))).is_err() {
                             #[cfg(feature = "internal_log")]
-                            log::warn!("[Demon] I worked the answer, but I could not send it back...");   
+                            log::warn!("[{}] demon handle result could not be delivered back", self.demon.id());   
                         }
                     } else {
                         if tx.send(Err(Error::WrongType)).is_err() {
                             #[cfg(feature = "internal_log")]
-                            log::warn!("[Demon] I worked the answer, but I could not send it back...");   
+                            log::warn!("[{}] demon handle received a wrong type and the message could not be delivered back", self.demon.id());   
                         }
                     }
                 } else {
                     #[cfg(feature = "internal_log")]
-                    log::info!("This demon is no longer needed, bye");
+                    log::info!("[{}] all channels to this demon are now closed (impossible)", self.demon.id());
                     break;
                 },
                 bytes_read = self.read_stream.read_buf(&mut buf) => if 0 != bytes_read.unwrap() {
@@ -77,10 +83,16 @@ impl<I: 'static + Send, O: 'static + Send, D: 'static + Demon<Input = I, Output
                             None
                         },
                         Err(WSError::Incomplete) => None,
-                        Err(e) => panic!("{}", e)
+                        Err(_e) => {
+                            #[cfg(feature = "internal_log")]
+                            log::debug!("[{}] error occured while parsing websockets frame, {}", self.demon.id(), _e);
+                            None
+                        }
                     };
 
                     if let Some(frame) = maybe_frame {
+                        #[cfg(feature = "internal_log")]
+                        log::debug!("[{}] received complete websockets message, processing", self.demon.id());
                         // We got a correct message, we clear the buffer
                         buf.clear();
                         // And call the handler
@@ -88,43 +100,54 @@ impl<I: 'static + Send, O: 'static + Send, D: 'static + Demon<Input = I, Output
                             self.demon.on_message(message).await;
                         }
                     } else {
-                        // Closing the connection in a nice way
-                        break;
+                        // More data needs to arrive to parse the message  properly
+                        continue;
                     }
                 } else {
                     // Closed connection!
                     if buf.is_empty() {
+                        // Half-clean exit
+                        #[cfg(feature = "internal_log")]
+                        log::info!("[{}] ws connection dropped", self.demon.id());
                         break;
                     } else {
-                        // connection reset by peer!
-                        log::debug!("connection reset by peer");
+                        // connection reset by peer! horrible connection
+                        log::debug!("[{}] connection reset by peer, with some data in the buffer", self.demon.id());
                         break
                     }
                 },
                 res = self.instructions.recv() => match res {
                     Some(instruction) => match instruction {
-                        MiniHellInstruction::Shutdown => break,
+                        MiniHellInstruction::Shutdown => {
+                            #[cfg(feature = "internal_log")]
+                            log::info!("[{}] shutdown signal received", self.demon.id());
+                            break
+                        },
                         MiniHellInstruction::Message(result_mailbox, message) => {
+                            #[cfg(feature = "internal_log")]
+                            log::debug!("[{}] received message, adding to the processing queue", self.demon.id());
                             if mailbox.send((result_mailbox, message)).is_err() {
                                 #[cfg(feature = "internal_log")]
-                                log::warn!("[Demon] Impossible error happened, could not process message...");   
+                                log::warn!("[{}] impossible error happened, could not send back message to itself!", self.demon.id());   
                             }
                         }
                     },
                     None => {
                         #[cfg(feature = "internal_log")]
-                        log::debug!("Instructions will no longer arrive to hosting hell, closing demon for address {}", self.location.address);
+                        log::info!("[{}] all channels to this demon are now closed", self.demon.id());
                         break;
                     }
                 }
             }
+            #[cfg(feature = "internal_log")]
+            log::debug!("[{}] end of await loop", self.demon.id());
         }
 
-        #[cfg(feature = "internal_log")]
-        log::debug!("Leaving mini hell");
-
         // The same, in reverse order
         self.demon.vanquished().await;
         self.demon.on_close().await;
+
+        #[cfg(feature = "internal_log")]
+        log::debug!("[{}] demon thread finished", self.demon.id());
     }
 }
\ No newline at end of file
